// Hook.cpp: implementation of the CHook class.
//
//////////////////////////////////////////////////////////////////////

/* Written by Amir Israeli January 2000,
   You may use this class , modify it , but dont remove this header.
   israelaq@walla.co.il  www.amir-israeli.tripod.com
   BASED on a GREAT class of Paul DiLascia PixieLib 1998  askpd@pobox.com
   (found within MSJ articles C++ Q&A) */

#include "stdafx.h"
#include "Hook.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHook::CHook()
{
	m_hHookedWnd = NULL;
	m_pOldWndProc = NULL;
	m_pNextHook = NULL;
}

CHook::~CHook()
{
	UnHook(); //unhook object
}

BOOL CHook::HookWnd(HWND hWnd, BOOL bHighPriority)
{
	if (!::IsWindow(hWnd))	return FALSE;
	m_hHookedWnd = hWnd; //setting hWnd 

	CHookWndMap& map = CHook::GetHookMap();
	CHook *pHookEntry=NULL;

	if (!map.Lookup( m_hHookedWnd, pHookEntry)) { //key not found
		m_pOldWndProc = (WNDPROC) ::SetWindowLong( m_hHookedWnd, GWL_WNDPROC, (DWORD)HookWndProc);
		map.SetAt(m_hHookedWnd,this);
		return TRUE;
	}

	ASSERT (pHookEntry!=NULL);//key was found 
	m_pOldWndProc = pHookEntry->m_pOldWndProc;

	if (bHighPriority) { //high piority : put in the start
		m_pNextHook = pHookEntry;
		map.SetAt(m_hHookedWnd,this);
	}
	else { // low priority : put in end of list
		while (pHookEntry->m_pNextHook)
			pHookEntry = pHookEntry->m_pNextHook;
		pHookEntry->m_pNextHook = this;
	}

	return (m_pOldWndProc!=NULL);
}

BOOL CHook::UnHookWnd(HWND m_hWnd)
{
	if (!::IsWindow(m_hWnd)) //no window
		return FALSE;

	CHookWndMap &map = GetHookMap();
	CHook *pHookEntry=NULL;
	if (!map.Lookup( m_hWnd, pHookEntry))
		return TRUE;//not found such window : so not hooked

	map.RemoveKey(m_hWnd);
	//assign the previous wndproc
	::SetWindowLong( m_hWnd, GWL_WNDPROC, (DWORD)pHookEntry->m_pOldWndProc);
	return TRUE;
}

//private to unhook 1 OBJECT only from hooking window messages
BOOL CHook::UnHook()
{
	CHook *pHookEntry=NULL;
	CHookWndMap &map = GetHookMap();
	if (!map.Lookup(m_hHookedWnd,pHookEntry))
		return TRUE;//not found such window : so not hooked

	CHook *pTmp=pHookEntry;
	for (int nCount=0; (pTmp) ; pTmp = pTmp->m_pNextHook, nCount++);

	if (nCount == 1) //only 1 object left
		return UnHookWnd(m_hHookedWnd);

	//more than 1 object ...
	if (pHookEntry == this) // in start
		map.SetAt(m_hHookedWnd,m_pNextHook);
	else { //in the middle or end
		while (pHookEntry->m_pNextHook != this)
			pHookEntry = pHookEntry->m_pNextHook;
		pHookEntry->m_pNextHook = pHookEntry->m_pNextHook->m_pNextHook;
	}

	return TRUE;
}

/* Window proc-like virtual function which specific CHook's will
 override to do stuff. Default passes the message to the next hook; 
 the last hook passes the message to the original window.
 You MUST call this at the end of your WindowProc if you want the real
 window to get the message. This is just like CWnd::WindowProc, except that
 a CHook is not a window.

  ... I called this function WindowProc2 instead of WindowProc
 In order that messages that go to a class that is derived from CWnd
 go to a different function than WindowProc : so a person could distinguish
 between messages of that window to messages of hooked window.
 (Amir Israeli 2000) */
LRESULT CHook::WindowProc2(UINT msg, WPARAM wp, LPARAM lp)
{
//	ASSERT_VALID(this);  // removed for speed
	ASSERT(m_pOldWndProc);
	return m_pNextHook ? m_pNextHook->WindowProc2(msg, wp, lp) :	
		::CallWindowProc(m_pOldWndProc, m_hHookedWnd, msg, wp, lp);
}

// call next hook in series of hooks to a window
LRESULT CHook::NextHook(UINT msg, WPARAM wp, LPARAM lp)
{
	ASSERT(m_pOldWndProc);
	return m_pNextHook ? m_pNextHook->WindowProc2(msg, wp, lp) :	
		::CallWindowProc(m_pOldWndProc, m_hHookedWnd, msg, wp, lp);
}


LRESULT CALLBACK
HookWndProc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{
#ifdef _USRDLL
	// If this is a DLL, need to set up MFC state
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
#endif

	// Set up MFC message state just in case anyone wants it
	// This is just like AfxCallWindowProc, but we can't use that because
	// a CSubclassWnd is not a CWnd.
	//
	MSG& curMsg = AfxGetThreadState()->m_lastSentMsg;
	MSG  oldMsg = curMsg;   // save for nesting
	curMsg.hwnd	= hwnd;
	curMsg.message = msg;
	curMsg.wParam  = wp;
	curMsg.lParam  = lp;

	LRESULT lr = 0;

	CHookWndMap& map = CHook::GetHookMap(); //get CHook relevant pointer to message
	CHook *pHook = NULL;
	map.Lookup( hwnd, pHook);
	
	ASSERT(pHook!=NULL); //if message here then hooked or ??? (another method ?)

	if (msg == WM_NCDESTROY) {
		// Window is being destroyed: unhook all hooks (for this window)
		// and pass msg to orginal window proc
		//
		WNDPROC wndproc = NULL;
		if (pHook) wndproc = pHook->m_pOldWndProc;
		CHook::UnHookWnd(hwnd);
		lr = ::CallWindowProc( wndproc, hwnd, msg, wp, lp);
	}
	else {
		// pass to msg hook
		lr = pHook->WindowProc2(msg, wp, lp);
	}

	curMsg = oldMsg; // pop state
	return lr;
}


