// Hook.h: interface for the CHook class.
//
//////////////////////////////////////////////////////////////////////

/* Written by Amir Israeli January 2000,
   You may use this class , modify it , but dont remove this header.
   israelaq@wall.co.il  www.amir-israeli.tripod.com
   BASED on a GREAT class of Paul DiLascia PixieLib 1998  askpd@pobox.com
   (found within MSJ articles C++ Q&A) 
*/

#if !defined(AFX_HOOK_H__D73DFAA1_8661_11D4_B8C4_C967DB3F4741__INCLUDED_)
#define AFX_HOOK_H__D73DFAA1_8661_11D4_B8C4_C967DB3F4741__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <afxtempl.h>

/* CHook class : (see more details at source code) CHook is based on
 Paul Dilascia's implementaion of CSubclassWnd but unlike his (GREAT) 
 class (As a matter of fact I always liked to look into his code !!! Thanks)
 which is mostly used in a CObject derived class (or some other manager class)
 This class is used in CWnd derived class. : 
 How to use :
*) derive a class from CWnd + CHook
*) override the function WindowProc2 if you want to use some messages
*) when you call HookWnd(...) , you hook a window messages and messages will come
to the WindowProc2 function : you can do anything with the messages
*) You can even call WindowProc(msg, wp, lp) from WindowProc2 to use messages
	as if they were from the same window . Why ? an example is in CAnimCursor 
	(a class of me) : this class shows an animation ccursor whenever any window (parent or children)
	are hooked and mouse moves : it also cancels the mouse SetCursor(NULL) in those windows.
	(hooks WM_SETCURSOR)
*) You can call the next hooks by several methods : (from WindowProc2)
	- CHook::WindowProc2(msg, wp, lp);
	- Next(msg, wp, lp);
	- NextHook(msg, wp, lp);
	I provided several names because calling the base isn't so nice ?!? (too many letters)
	I like to use "Next(msg, wp, lp)" as you see in this class.
*) You may unhook a window (automaticly on OnDestroy)

class CMyWindow : public CWnd , public CHook
{
	// constructors , destructors
	...

	LRESULT WindowProc2(UINT msg, WPARAM wp, LPARAM lp)
	{
		switch (msg)
		{
		case WM_MOUSEMOVE:
			{
				// doing something with the message

			}
	
		}

		return 0;
	}
	
	return Next(msg, wp, lp); // call the next hook
}

*/

class CHook;
typedef CMap<HWND,HWND,CHook*,CHook*> CHookWndMap;

class CHook  
{
public:
	CHook();
	virtual ~CHook();

	BOOL IsHooked()	{ return ::IsWindow(m_hHookedWnd); }
	BOOL UnHook();

	BOOL HookWnd(HWND m_hWnd, BOOL bHighPriority=TRUE);
	BOOL HookWnd(CWnd *pWnd, BOOL bHighPriority=TRUE) {
		if (!pWnd)	return FALSE; //no window
		return HookWnd(pWnd->GetSafeHwnd(),bHighPriority);
	}
	
	static BOOL UnHookWnd(HWND m_hWnd);
	static BOOL UnHookWnd(CWnd *pWnd) {
		if (!pWnd)	return FALSE; //no window
		return UnHookWnd(pWnd->GetSafeHwnd());
	}
	
protected:
	static CHookWndMap& GetHookMap() {
		static CHookWndMap map;
		return map;
	}

	// Override this to handle messages in specific handlers
	virtual LRESULT WindowProc2(UINT msg, WPARAM wp, LPARAM lp);
	// call next hook in series of hooks to a window
	LRESULT NextHook(UINT msg, WPARAM wp, LPARAM lp);
	LRESULT Next(UINT msg, WPARAM wp, LPARAM lp)
		{	return NextHook(msg, wp, lp); }

	friend LRESULT CALLBACK HookWndProc(HWND, UINT, WPARAM, LPARAM);

protected:
	HWND  m_hHookedWnd;
	WNDPROC  m_pOldWndProc;
	CHook *m_pNextHook;
};

#endif // !defined(AFX_HOOK_H__D73DFAA1_8661_11D4_B8C4_C967DB3F4741__INCLUDED_)


