// StatusBarWndInsertor.cpp: implementation of the CStatusBarWndInsertor class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "StatusBarWndInsertor.h"

/* Written by Amir Israeli January 2000,
   You may use this class , modify it , but dont remove this header.
   israelaq@wall.co.il  www.amir-israeli.tripod.com  */

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CStatusBarWndInsertor::CStatusBarWndInsertor()
{
}

CStatusBarWndInsertor::~CStatusBarWndInsertor()
{
}
void CStatusBarWndInsertor::RemoveAll()
{
	int nCount = m_listWndPanes.GetCount();
	if (!nCount) return;

	POSITION pos = m_listWndPanes.GetHeadPosition();
	while (pos)
		delete m_listWndPanes.GetNext(pos);
}

BOOL CStatusBarWndInsertor::SetStatusBar(CStatusBar *m_pStatusBar)
{
	return (m_pStatusBar) ?
		SetStatusBar(m_pStatusBar->GetSafeHwnd()) : FALSE;
}
BOOL CStatusBarWndInsertor::SetStatusBar(HWND m_hStatusBar)
{
	if (!::IsWindow(m_hStatusBar))	
		return FALSE;
	return HookWnd(m_hStatusBar,TRUE);
}

BOOL CStatusBarWndInsertor::AddWindow(CWnd *m_pWnd,int nIndex)
{
	if (!m_pWnd || !::IsWindow(m_pWnd->GetSafeHwnd()) || nIndex<0)
		return FALSE;
	if (FindWindow(m_pWnd->GetSafeHwnd()))
		return FALSE; // already exist
	if (FindWindow(nIndex))
		return FALSE; // pane already occupied by other window
	CPaneWnd *pPane = new CPaneWnd;
	if (!pPane) return FALSE;

	m_pWnd->ModifyStyleEx(0,WS_EX_STATICEDGE,SWP_DRAWFRAME);
	pPane->nIndex = nIndex;
	pPane->m_hWnd= m_pWnd->GetSafeHwnd();

	::SetParent(pPane->m_hWnd,m_hHookedWnd);
	m_listWndPanes.AddHead(pPane);

	CRect rc;
	if (::SendMessage(m_hHookedWnd,SB_GETRECT,(WPARAM)nIndex,(LPARAM)&rc))
		::MoveWindow(pPane->m_hWnd,rc.left,rc.top,rc.Width(),rc.Height(),TRUE);

	return TRUE;
}
void CStatusBarWndInsertor::RemoveWindow(CWnd *m_pWnd)
{
	if (m_pWnd && m_pWnd->GetSafeHwnd()) 
		RemoveWindow(m_pWnd->GetSafeHwnd());
}
void CStatusBarWndInsertor::RemoveWindow(int nIndex)
{
	POSITION pos = FindWindow(nIndex);
	RemoveAt(pos);
}
void CStatusBarWndInsertor::RemoveWindow(HWND m_hWnd)
{
	POSITION pos = FindWindow(m_hWnd);
	RemoveAt(pos);
}

void CStatusBarWndInsertor::RemoveAt(POSITION pos)
{
	if (!pos) return;
	CPaneWnd *pPane = m_listWndPanes.GetAt(pos);
	m_listWndPanes.RemoveAt(pos);
	if (pPane) {
		HWND hWnd = pPane->m_hWnd;
		if (::IsWindow(hWnd))
			::ShowWindow(hWnd,SW_HIDE);
		delete pPane;
	}
}


// These 2 functions find an item in the list of inserted windows
// into status bar (looks by pane index/by window handle)
POSITION CStatusBarWndInsertor::FindWindow(HWND m_hWnd)
{
	int nCount = m_listWndPanes.GetCount();
	if (!nCount) return 0;
	POSITION pos = m_listWndPanes.GetHeadPosition();
	CPaneWnd *pPane;
	while (pos) {
		pPane = m_listWndPanes.GetNext(pos);
		if (pPane && pPane->m_hWnd==m_hWnd)
			return pos;
	}
	return 0;
}
POSITION CStatusBarWndInsertor::FindWindow(int nIndex)
{
	int nCount = m_listWndPanes.GetCount();
	if (!nCount) return 0;
	POSITION pos = m_listWndPanes.GetHeadPosition();
	CPaneWnd *pPane;
	while (pos) {
		pPane = m_listWndPanes.GetNext(pos);
		if (pPane && pPane->nIndex==nIndex)
			return pos;
	}
	return 0;
}

// reposision all windows in areas of panes due to state change
// (to visible) or due to change in size
void CStatusBarWndInsertor::RepositionWindows()
{
	int nCount = m_listWndPanes.GetCount();
	if (!nCount) return;

	POSITION pos = m_listWndPanes.GetHeadPosition();
	CPaneWnd *pPane;
	while (pos) 
	{
		pPane = m_listWndPanes.GetNext(pos);
		if (pPane) {
			int nIndex = pPane->nIndex;
			HWND hWnd = pPane->m_hWnd;
			if (::IsWindow(hWnd) && ::IsWindow(m_hHookedWnd))
			{
				CRect rc;
				if (::SendMessage(m_hHookedWnd,SB_GETRECT,(WPARAM)nIndex,(LPARAM)&rc))
					::MoveWindow(hWnd,rc.left,rc.top,rc.Width(),rc.Height(),TRUE);
			}
		} // pPane!=NULL	
	}
}

// messages of status bar
LRESULT CStatusBarWndInsertor::WindowProc2(UINT msg, WPARAM wp, LPARAM lp)
{
	LRESULT lr = 0;
	switch (msg)
	{
	case WM_SIZE:
		{
			// update state or size of window
			lr = Next(msg, wp, lp);	
			// re-position all windows in place
			RepositionWindows();
			return lr;
		}
		break;

	case WM_DESTROY: 
		// when destroying status bar remove all windows
		RemoveAll();
		break;
	}

	// always give the message to the status bar
	return Next(msg, wp, lp);
}




